@@ -39,15 +39,20 @@
 
 (defvar compilation-error-list nil
   "List of error message descriptors for visiting erring functions.
-Each error descriptor is a cons (or nil).  Its car is a marker pointing to
-an error message.  If its cdr is a marker, it points to the text of the
-line the message is about.  If its cdr is a cons, that cons's car is a cons
-\(DIRECTORY . FILE\), specifying the file the message is about, and its cdr
-is the number of the line the message is about.  Or its cdr may be nil if
-that error is not interesting.
+Each error descriptor is a cons (or nil).  Its car is a marker
+pointing to an error message.  If its cdr is a marker, it points to
+the text of the line the message is about.  If its cdr is a cons, that
+cons's car is the name of the file the message is about, and its cdr
+is the number of the line the message is about.  Or its cdr may be nil
+if that error is not interesting.
 
 The value may be t instead of a list; this means that the buffer of
-error messages should be reparsed the next time the list of errors is wanted.")
+error messages should be reparsed the next time the list of errors is wanted.
+
+Some other commands (like `diff') use this list to control the error
+message tracking facilites; if you change its structure, you should make
+sure you also change those packages.  Perhaps it is better not to change
+it at all.")
 
 (defvar compilation-old-error-list nil
   "Value of `compilation-error-list' after errors were parsed.")
@@ -108,27 +113,35 @@ or when it is used with \\[next-error] or \\[compile-goto-error].")
     ;; We'll insist that the number be followed by a colon or closing
     ;; paren, because otherwise this matches just about anything
     ;; containing a number with spaces around it.
-    ("^\\([^:( \t\n]+\\)[ \t]*[:(][ \t]*\\([0-9]+\\)[:) \t]" 1 2)
+    ("\n\\([^:( \t\n]+\\)[:(][ \t]*\\([0-9]+\\)[:) \t]" 1 2)
 
     ;; 4.3BSD lint pass 2
     ;; 	strcmp: variable # of args. llib-lc(359)  ::  /usr/src/foo/foo.c(8)
-    ("[ \t:]+\\([^:( \t\n]+\\)[ \t]*[:(]*(+[ \t]*\\([0-9]+\\))[:) \t]*$" 1 2)
+    ("[ \t:]\\([^:( \t\n]+\\)[:(](+[ \t]*\\([0-9]+\\))[:) \t]*$" 1 2)
 
     ;; 4.3BSD lint pass 3
     ;; 	bloofle defined( /users/wolfgang/foo.c(4) ), but never used
     ;; This used to be
     ;; ("[ \t(]+\\([^:( \t\n]+\\)[:( \t]+\\([0-9]+\\)[:) \t]+" 1 2)
     ;; which is regexp Impressionism - it matches almost anything!
-    ("([ \t]*\\([^:( \t\n]+\\)[ \t]*[:(][ \t]*\\([0-9]+\\))" 1 2)
+    ("([ \t]*\\([^:( \t\n]+\\)[:(][ \t]*\\([0-9]+\\))" 1 2)
 
-    ;; Line 45 of "foo.c": bloofel undefined (who does this?)
-    ("^[Ll]ine[ \t]+\\([0-9]+\\)[ \t]+of[ \t]+\"\\([^\"\n]+\\)\":" 2 1)
+    ;; Ultrix 3.0 f77:
+    ;;  Error on line 3 of t.f: Execution error unclassifiable statement    
+    ;; Unknown who does this:
+    ;;  Line 45 of "foo.c": bloofel undefined
+    ("\n\\(Error on \\)?[Ll]ine[ \t]+\\([0-9]+\\)[ \t]+\
+of[ \t]+\"?\\([^\"\n]+\\)\"?:" 3 2)
 
     ;; Apollo cc, 4.3BSD fc:
     ;;	"foo.f", line 3: Error: syntax error near end of statement
-    ;; or MIPS RISC CC - the one distributed with Ultrix:
+    ;; IBM RS6000:
+    ;;  "vvouch.c", line 19.5: 1506-046 (S) Syntax error.
+    ("\"\\([^,\" \n\t]+\\)\", line \\([0-9]+\\)[:.]" 1 2)
+
+    ;; MIPS RISC CC - the one distributed with Ultrix:
     ;;	ccom: Error: foo.c, line 2: syntax error
-    ("\\b\"?\\([^,\" \n\t]+\\)\"?, line \\([0-9]+\\):" 1 2)
+    ("rror: \\([^,\" \n\t]+\\), line \\([0-9]+\\):" 1 2)
 
     ;; IBM AIX PS/2 C version 1.1:
     ;;	****** Error number 140 in line 8 of file errors.c ******
@@ -294,36 +307,37 @@ Returns the compilation buffer created."
 	  (goto-char (point-max)))
       ;; Pop up the compilation buffer.
       (setq outwin (display-buffer outbuf))
-      (set-buffer outbuf)
-      (compilation-mode)
-      (buffer-disable-undo (current-buffer))
-      (setq buffer-read-only t)
-      (set (make-local-variable 'compilation-parse-errors-function) parser)
-      (set (make-local-variable 'compilation-error-message) error-message)
-      (set (make-local-variable 'compilation-error-regexp-alist) regexp-alist)
-      (setq default-directory thisdir
-	    compilation-directory-stack (list default-directory))
-      (set-window-start outwin (point-min))
-      (setq mode-name name-of-mode)
-      (or (eq outwin (selected-window))
-	  (set-window-point outwin (point-min)))
-      (and compilation-window-height
-	   (= (window-width outwin) (frame-width))
-	   (let ((w (selected-window)))
-	     (unwind-protect
-		 (progn
-		   (select-window outwin)
-		   (enlarge-window (- compilation-window-height
-				      (window-height))))
-	       (select-window w))))
-      ;; Start the compilation.
-      (let ((proc (start-process-shell-command (downcase mode-name)
-					       outbuf
-					       command)))
-	(set-process-sentinel proc 'compilation-sentinel)
-	(set-process-filter proc 'compilation-filter)
-	(set-marker (process-mark proc) (point) outbuf)
-	(setq compilation-in-progress (cons proc compilation-in-progress))))
+      (save-excursion
+	(set-buffer outbuf)
+	(compilation-mode)
+	(buffer-disable-undo (current-buffer))
+	(setq buffer-read-only t)
+	(set (make-local-variable 'compilation-parse-errors-function) parser)
+	(set (make-local-variable 'compilation-error-message) error-message)
+	(set (make-local-variable 'compilation-error-regexp-alist) regexp-alist)
+	(setq default-directory thisdir
+	      compilation-directory-stack (list default-directory))
+	(set-window-start outwin (point-min))
+	(setq mode-name name-of-mode)
+	(or (eq outwin (selected-window))
+	    (set-window-point outwin (point-min)))
+	(and compilation-window-height
+	     (= (window-width outwin) (frame-width))
+	     (let ((w (selected-window)))
+	       (unwind-protect
+		   (progn
+		     (select-window outwin)
+		     (enlarge-window (- compilation-window-height
+					(window-height))))
+		 (select-window w))))
+	;; Start the compilation.
+	(let ((proc (start-process-shell-command (downcase mode-name)
+						 outbuf
+						 command)))
+	  (set-process-sentinel proc 'compilation-sentinel)
+	  (set-process-filter proc 'compilation-filter)
+	  (set-marker (process-mark proc) (point) outbuf)
+	  (setq compilation-in-progress (cons proc compilation-in-progress)))))
     ;; Make it so the next C-x ` will use this buffer.
     (setq compilation-last-buffer outbuf)))
 
@@ -497,8 +511,7 @@ Does NOT find the source line like \\[next-error]."
   (setq data (cdr data))
   (if (markerp data)
       (buffer-file-name (marker-buffer data))
-    (setq data (car data))
-    (expand-file-name (cdr data) (car data))))
+    (car data)))
 
 (defun compilation-next-file (n)
   "Move point to the next error for a different file than the current one."
@@ -728,7 +741,10 @@ See variables `compilation-parse-errors-function' and
 		(or (markerp (cdr next-error))
 		    ;; This error has a filename/lineno pair.
 		    ;; Find the file and turn it into a marker.
-		    (let* ((fileinfo (car (cdr next-error)))
+		    (let* ((fileinfo
+			    (cons (file-name-directory (car (cdr next-error)))
+				  (file-name-nondirectory
+				   (car (cdr next-error)))))
 			   (buffer (compilation-find-file (cdr fileinfo)
 							  (car fileinfo)
 							  (car next-error))))
@@ -890,7 +906,7 @@ See variables `compilation-parse-errors-function' and
 See variable `compilation-parse-errors-function' for the interface it uses."
   (setq compilation-error-list nil)
   (message "Parsing error messages...")
-  (let (text-buffer
+  (let (text-buffer orig orig-expanded parent-expanded
 	regexp enter-group leave-group error-group
 	alist subexpr error-regexp-groups
 	(found-desired nil)
@@ -901,7 +917,12 @@ See variable `compilation-parse-errors-function' for the interface it uses."
     ;; Don't parse the first two lines as error messages.
     ;; This matters for grep.
     (if (bobp)
-	(forward-line 2))
+	(progn
+	  (forward-line 2)
+	  ;; Move back so point is before the newline.
+	  ;; This matters because some error regexps use \n instead of ^
+	  ;; to be faster.
+	  (forward-char -1)))
 
     ;; Compile all the regexps we want to search for into one.
     (setq regexp (concat "\\(" compilation-enter-directory-regexp "\\)\\|"
@@ -940,6 +961,10 @@ See variable `compilation-parse-errors-function' for the interface it uses."
       (setq subexpr (+ subexpr 1 (count-regexp-groupings (car (car alist)))))
       (setq alist (cdr alist)))
 
+    (setq orig default-directory)
+    (setq orig-expanded (file-truename orig))
+    (setq parent-expanded (expand-file-name "../" orig-expanded))
+
     (while (and (not found-desired)
 		;; We don't just pass LIMIT-SEARCH to re-search-forward
 		;; because we want to find matches containing LIMIT-SEARCH
@@ -954,6 +979,12 @@ See variable `compilation-parse-errors-function' for the interface it uses."
 		     (expand-file-name
 		      (buffer-substring (match-beginning (+ enter-group 1))
 					(match-end (+ enter-group 1)))))))
+	       ;; The directory name in the "entering" message
+	       ;; is a truename.  Try to convert it to a form
+	       ;; like what the user typed in.
+	       (setq dir
+		     (compile-abbreviate-directory dir orig orig-expanded
+						   parent-expanded))
 	       (setq compilation-directory-stack
 		     (cons dir compilation-directory-stack))
 	       (and (file-directory-p dir)
@@ -970,6 +1001,12 @@ See variable `compilation-parse-errors-function' for the interface it uses."
 			    (buffer-substring beg
 					      (match-end (+ leave-group
 							    1)))))))
+		     ;; The directory name in the "entering" message
+		     ;; is a truename.  Try to convert it to a form
+		     ;; like what the user typed in.
+		     (setq dir
+			   (compile-abbreviate-directory dir orig orig-expanded
+							 parent-expanded))
 		     (while (and stack
 				 (not (string-equal (car stack) dir)))
 		       (setq stack (cdr stack)))))
@@ -993,9 +1030,12 @@ See variable `compilation-parse-errors-function' for the interface it uses."
 	     ;; Extract the file name and line number from the error message.
 	     (let ((beginning-of-match (match-beginning 0)) ;looking-at nukes
 		   (filename
-		    (cons default-directory
-			  (buffer-substring (match-beginning (nth 1 alist))
-					    (match-end (nth 1 alist)))))
+		    (save-excursion
+		      (goto-char (match-end (nth 1 alist)))
+		      (skip-chars-backward " \t")
+		      (let ((name (buffer-substring (match-beginning (nth 1 alist))
+						    (point))))
+			(expand-file-name name default-directory))))
 		   (linenum (save-restriction
 			      (narrow-to-region
 			       (match-beginning (nth 2 alist))
@@ -1054,6 +1094,28 @@ See variable `compilation-parse-errors-function' for the interface it uses."
   (setq compilation-error-list (nreverse compilation-error-list))
   (message "Parsing error messages...done"))
 
+;; If directory DIR is a subdir of ORIG or of ORIG's parent,
+;; return a relative name for it starting from ORIG or its parent.
+;; ORIG-EXPANDED is an expanded version of ORIG.
+;; PARENT-EXPANDED is an expanded version of ORIG's parent.
+;; Those two args could be computed here, but we run faster by
+;; having the caller compute them just once.
+(defun compile-abbreviate-directory (dir orig orig-expanded parent-expanded)
+  (if (and (> (length dir) (length orig-expanded))
+	   (string= orig-expanded
+		    (substring dir 0 (length orig-expanded))))
+      (setq dir
+	    (concat orig
+		    (substring dir (length orig-expanded)))))
+  (if (and (> (length dir) (length parent-expanded))
+	   (string= parent-expanded
+		    (substring dir 0 (length parent-expanded))))
+    (setq dir
+	  (concat (file-name-directory
+		   (directory-file-name orig))
+		  (substring dir (length parent-expanded)))))
+  dir)
+
 (provide 'compile)
 
 ;;; compile.el ends here
