@@ -141,7 +141,7 @@ One argument, the tag info returned by `snarf-tag-function'.")
 (defvar tags-included-tables-function nil
   "Function to do the work of `tags-included-tables' (which see).")
 (defvar verify-tags-table-function nil
-  "Function to return t iff the current buffer vontains a valid
+  "Function to return t iff the current buffer contains a valid
 \(already initialized\) tags file.")
 
 ;; Initialize the tags table in the current buffer.
@@ -636,7 +636,7 @@ See documentation of variable `tags-file-name'."
 		;; Move to the saved location.
 		(set-buffer (marker-buffer marker))
 	      (goto-char (marker-position marker))
-	      ;; Kill that marker so it doesn't slow down editting.
+	      ;; Kill that marker so it doesn't slow down editing.
 	      (set-marker marker nil nil)
 	      ;; Run the user's hook.  Do we really want to do this for pop?
 	      (run-hooks 'local-find-tag-hook))))
@@ -879,7 +879,7 @@ See documentation of variable `tags-file-name'."
 		 (goto-tag-location-function . etags-goto-tag-location)
 		 (find-tag-regexp-search-function . re-search-forward)
 		 (find-tag-regexp-tag-order . (tag-re-match-p))
-		 (find-tag-regexp-next-line-after-failuire-p . t)
+		 (find-tag-regexp-next-line-after-failure-p . t)
 		 (find-tag-search-function . search-forward)
 		 (find-tag-tag-order . (tag-exact-match-p tag-word-match-p
 							  tag-any-match-p))
@@ -915,7 +915,8 @@ See documentation of variable `tags-file-name'."
       ;;   \6 is not interesting;
       ;;   \7 is the explicitly-specified tag name.
       (while (re-search-forward
-	      "^\\(\\(.+[ \t]+\\)?\\([-a-zA-Z0-9_$]+\\)[^-a-zA-Z0-9_$]*\\)\177\
+	      "^\\(\\(.+[^-a-zA-Z0-9_$]+\\)?\\([-a-zA-Z0-9_$]+\\)\
+\[^-a-zA-Z0-9_$]*\\)\177\
 \\([0-9]+\\),\\([0-9]+\\)\\(,\001\\([^\n]+\\)\\)?\n"
 	      nil t)
 	(intern	(if (match-beginning 6)
