@@ -45,7 +45,7 @@ Delete these headers from old message when it's inserted in a reply.")
 
 ;; Useful to set in site-init.el
 ;;;###autoload
-(defconst send-mail-function 'sendmail-send-it "\
+(defvar send-mail-function 'sendmail-send-it "\
 Function to call to send the current buffer as mail.
 The headers are be delimited by a line which is `mail-header-separator'.")
 
@@ -76,6 +76,26 @@ The alias definitions in `~/.mailrc' have this form:
 (defvar mail-yank-prefix nil
   "*Prefix insert on lines of yanked message being replied to.
 nil means use indentation.")
+(defvar mail-indentation-spaces 3
+  "*Number of spaces to insert at the beginning of each cited line.
+Used by `mail-yank-original' via `mail-yank-cite'.")
+(defvar mail-yank-hooks '(mail-indent-citation)
+  "Obsolete hook for modifying a citation just inserted in the mail buffer.
+Each hook function can find the citation between (point) and (mark t).
+And each hook function should leave point and mark around the citation
+text as modified.
+
+This is a normal hook, misnamed for historical reasons.
+It is semi-obsolete and mail agents should no longer use it.")
+
+(defvar mail-citation-hook nil
+  "*Hook for modifying a citation just inserted in the mail buffer.
+Each hook function can find the citation between (point) and (mark t).
+And each hook function should leave point and mark around the citation
+text as modified.
+
+If this hook is entirely empty (nil), a default action is taken
+instead of no action.")
 
 (defvar mail-abbrevs-loaded nil)
 (defvar mail-mode-map nil)
@@ -196,6 +216,8 @@ C-c C-v  mail-sent-via (add a sent-via field for each To or CC)."
 				   "$\\|^[ \t]*[-_][-_][-_]+$\\|"
 				   paragraph-separate))
   (run-hooks 'text-mode-hook 'mail-mode-hook))
+
+;;; Set up keymap.
 
 (if mail-mode-map
     nil
@@ -213,21 +235,79 @@ C-c C-v  mail-sent-via (add a sent-via field for each To or CC)."
   (define-key mail-mode-map "\C-c\C-v" 'mail-sent-via)
   (define-key mail-mode-map "\C-c\C-c" 'mail-send-and-exit)
   (define-key mail-mode-map "\C-c\C-s" 'mail-send))
+
+(define-key mail-mode-map [menu-bar mail]
+  (cons "Mail" (make-sparse-keymap "Mail")))
+
+(define-key mail-mode-map [menu-bar mail fill]
+  '("Fill Citation" . mail-fill-yanked-message))
+
+(define-key mail-mode-map [menu-bar mail yank]
+  '("Cite Original" . mail-yank-original))
+
+(define-key mail-mode-map [menu-bar mail signature]
+  '("Insert Signature" . mail-signature))
+
+(define-key mail-mode-map [menu-bar mail cancel]
+  '("Cancel" . mail-dont-send))
+
+(define-key mail-mode-map [menu-bar mail send-stay]
+  '("Send, Keep Editing" . mail-send))
+
+(define-key mail-mode-map [menu-bar mail send]
+  '("Send Message" . mail-send-and-exit))
+
+(define-key mail-mode-map [menu-bar headers]
+  (cons "Headers" (make-sparse-keymap "Headers")))
+
+(define-key mail-mode-map [menu-bar headers sent-via]
+  '("Sent Via" . mail-sent-via))
+
+(define-key mail-mode-map [menu-bar headers text]
+  '("Text" . mail-text))
+
+(define-key mail-mode-map [menu-bar headers bcc]
+  '("Bcc" . mail-bcc))
+
+(define-key mail-mode-map [menu-bar headers fcc]
+  '("Fcc" . mail-fcc))
+
+(define-key mail-mode-map [menu-bar headers cc]
+  '("Cc" . mail-cc))
+
+(define-key mail-mode-map [menu-bar headers subject]
+  '("Subject" . mail-subject))
+
+(define-key mail-mode-map [menu-bar headers to]
+  '("To" . mail-to))
 
 (defun mail-send-and-exit (arg)
   "Send message like `mail-send', then, if no errors, exit from mail buffer.
 Prefix arg means don't delete this window."
   (interactive "P")
   (mail-send)
+  (mail-bury arg))
+
+(defun mail-dont-send (arg)
+  "Don't send the message you have been editing.
+Prefix arg means don't delete this window."
+  (interactive "P")
+  (mail-bury arg))
+
+(defun mail-bury (arg)
+  "Bury this mail buffer."
   (let ((newbuf (other-buffer (current-buffer))))
     (bury-buffer (current-buffer))
-    (if (and (not arg)
-	     (not (one-window-p))
-	     (save-excursion
-	       (set-buffer (window-buffer (next-window (selected-window) 'not)))
-	       (eq major-mode 'rmail-mode)))
-	(delete-window)
-      (switch-to-buffer newbuf))))
+    (if (and (cdr (assq 'dedicated (frame-parameters)))
+	     (not (null (delq (selected-frame) (visible-frame-list)))))
+	(delete-frame (selected-frame))
+      (if (and (not arg)
+	       (not (one-window-p))
+	       (save-excursion
+		 (set-buffer (window-buffer (next-window (selected-window) 'not)))
+		 (eq major-mode 'rmail-mode)))
+	  (delete-window)
+	(switch-to-buffer newbuf)))))
 
 (defun mail-send ()
   "Send the message in the current buffer.
@@ -236,8 +316,10 @@ or error messages, and inform user.
 Otherwise any failure is reported in a message back to
 the user from the mailer."
   (interactive)
-  (if (or (buffer-modified-p)
-          (y-or-n-p "Message already sent; resend? "))
+  (if (if buffer-file-name
+	  (y-or-n-p "Send buffer contents as mail message? ")
+	(or (buffer-modified-p)
+	    (y-or-n-p "Message already sent; resend? ")))
       (progn
 	(message "Sending...")
 	(run-hooks 'mail-send-hook)
@@ -356,7 +438,6 @@ the user from the mailer."
   (let (fcc-list
 	(rmailbuf (current-buffer))
 	(time (current-time))
-	timezone
 	(tembuf (generate-new-buffer " rmail output"))
 	(case-fold-search t))
     (save-excursion
@@ -370,21 +451,16 @@ the user from the mailer."
 			     fcc-list))
 	(delete-region (match-beginning 0)
 		       (progn (forward-line 1) (point))))
-      (let* ((foo (current-time-zone time))
-	     (offset (if (car foo) (/ (car foo) 60) 0))
-	     (abs (abs offset)))
-	(setq timezone (format "%s%02d%02d"
-			       (if (< offset 0) "-" "+")
-			       (/ abs 60)
-			       (% abs 60))))
       (set-buffer tembuf)
       (erase-buffer)
-      (insert "From " (user-login-name) " "
+      ;; This initial newline is written out if the fcc file already exists.
+      (insert "\nFrom " (user-login-name) " "
 	      (current-time-string time) "\n")
       ;; Insert the time zone before the year.
       (forward-char -1)
       (forward-word -1)
-      (insert timezone " ")
+      (require 'mail-utils)
+      (insert (mail-rfc822-time-zone time) " ")
       (goto-char (point-max))
       (insert-buffer-substring rmailbuf)
       ;; Make sure messages are separated.
@@ -403,7 +479,9 @@ the user from the mailer."
 	  (if buffer
 	      ;; File is present in a buffer => append to that buffer.
 	      (let ((curbuf (current-buffer))
-		    (beg (point-min)) (end (point-max)))
+		    (beg (point-min)) (end (point-max))
+		    (beg2 (save-excursion (goto-char (point-min))
+					  (forward-line 2) (point))))
 		(save-excursion
 		  (set-buffer buffer)
 		  ;; Keep the end of the accessible portion at the same place
@@ -411,28 +489,39 @@ the user from the mailer."
 		  (let ((max (if (/= (1+ (buffer-size)) (point-max))
 				 (point-max))))
 		    (unwind-protect
-			(progn
-			  (narrow-to-region (point-min) (1+ (buffer-size)))
-			  (goto-char (point-max))
-			  (if (eq major-mode 'rmail-mode)
-			      ;; Append as a message to an RMAIL file
-			      (let ((buffer-read-only nil))
-				;; This forces RMAIL's message counters to be
-				;; recomputed when the next RMAIL operation is
-				;; done on the buffer.
-				;; See rmail-maybe-set-message-counters.
-				(setq rmail-total-messages nil)
+			;; Code below lifted from rmailout.el
+			;; function rmail-output-to-rmail-file:
+			(let ((buffer-read-only nil)
+			      (msg (and (boundp 'rmail-current-message)
+					rmail-current-message)))
+			  ;; If MSG is non-nil, buffer is in RMAIL mode.
+			  (if msg
+			      (progn
+				(rmail-maybe-set-message-counters)
+				(widen)
+				(narrow-to-region (point-max) (point-max))
 				(insert "\C-l\n0, unseen,,\n*** EOOH ***\n"
 					"From: " (user-login-name) "\n"
-					"Date: " (current-time-string) "\n")
-				(insert-buffer-substring curbuf beg end)
+					"Date: " (mail-rfc822-date) "\n")
+				(insert-buffer-substring curbuf beg2 end)
 				(insert "\n\C-_")
-				(rmail-set-message-counters))
+				(goto-char (point-min))
+				(widen)
+				(search-backward "\n\^_")
+				(narrow-to-region (point) (point-max))
+				(rmail-count-new-messages t)
+				(rmail-show-message msg)
+				(setq max nil))
+			    ;; Output file not in rmail mode
+			    ;; => just insert at the end.
+			    (narrow-to-region (point-min) (1+ (buffer-size)))
+			    (goto-char (point-max))
 			    (insert-buffer-substring curbuf beg end)))
 		      (if max (narrow-to-region (point-min) max))))))
 	    ;; Else append to the file directly.
 	    (write-region
 	     ;; Include a blank line before if file already exists.
+
 	     (if (file-exists-p (car fcc-list)) (point-min) (1+ (point-min)))
 	     (point-max) (car fcc-list) t)))
 	(setq fcc-list (cdr fcc-list))))
@@ -536,7 +625,7 @@ the user from the mailer."
     (end-of-line)
     (or atpoint
 	(delete-region (point) (point-max)))
-    (insert "\n\n--\n")
+    (insert "\n\n")
     (insert-file-contents (expand-file-name "~/.signature"))))
 
 (defun mail-fill-yanked-message (&optional justifyp)
@@ -551,6 +640,23 @@ Numeric argument means justify as well."
 				justifyp
 				t)))
 
+(defun mail-indent-citation ()
+  "Modify text just inserted from a message to be cited.
+The inserted text should be the region.
+When this function returns, the region is again around the modified text.
+
+Normally, indent each nonblank line `mail-indentation-spaces' spaces.
+However, if `mail-yank-prefix' is non-nil, insert that prefix on each line."
+  (let ((start (point)))
+    (mail-yank-clear-headers start (mark t))
+    (if (null mail-yank-prefix)
+	(indent-rigidly start (mark t) mail-indentation-spaces)
+      (save-excursion
+	(goto-char start)
+	(while (< (point) (mark t))
+	  (insert mail-yank-prefix)
+	  (forward-line 1))))))
+
 (defun mail-yank-original (arg)
   "Insert the message being replied to, if any (in rmail).
 Puts point before the text and mark after.
@@ -566,15 +672,12 @@ and don't delete any header fields."
 	(insert-buffer mail-reply-buffer)
 	(if (consp arg)
 	    nil
-	  (mail-yank-clear-headers start (mark t))
-	  (if (null mail-yank-prefix)
-	      (indent-rigidly start (mark t)
-			      (if arg (prefix-numeric-value arg) 3))
-	    (save-excursion
-	      (goto-char start)
-	      (while (< (point) (mark t))
-		(insert mail-yank-prefix)
-		(forward-line 1)))))
+	  (goto-char start)
+	  (let ((mail-indentation-spaces (if arg (prefix-numeric-value arg)
+					   mail-indentation-spaces)))
+	    (if mail-citation-hook
+		(run-hooks 'mail-citation-hook)
+	      (run-hooks 'mail-yank-hooks))))
 	;; This is like exchange-point-and-mark, but doesn't activate the mark.
 	;; It is cleaner to avoid activation, even though the command
 	;; loop would deactivate the mark because we inserted text.
