@@ -1,26 +1,24 @@
 ;;; x-win.el --- parse switches controlling interface with X window system
+;; Copyright (C) 1993 Free Software Foundation, Inc.
 
 ;; Author: FSF
 ;; Keywords: terminals
 
-;; Copyright (C) 1993 Free Software Foundation, Inc.
-
-;; This file is part of GNU Emacs.
-
-;; GNU Emacs is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY.  No author or distributor
-;; accepts responsibility to anyone for the consequences of using it
-;; or for whether it serves any particular purpose or works at all,
-;; unless he says so in writing.  Refer to the GNU Emacs General Public
-;; License for full details.
-
-;; Everyone is granted permission to copy, modify and redistribute
-;; GNU Emacs, but only under the conditions described in the
-;; GNU Emacs General Public License.   A copy of this license is
-;; supposed to have been given to you along with GNU Emacs so you
-;; can know your rights and responsibilities.  It should be in a
-;; file named COPYING.  Among other things, the copyright notice
-;; and this notice must be preserved on all copies.
+;;; This file is part of GNU Emacs.
+;;;
+;;; GNU Emacs is free software; you can redistribute it and/or modify
+;;; it under the terms of the GNU General Public License as published by
+;;; the Free Software Foundation; either version 2, or (at your option)
+;;; any later version.
+;;;
+;;; GNU Emacs is distributed in the hope that it will be useful,
+;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;;; GNU General Public License for more details.
+;;;
+;;; You should have received a copy of the GNU General Public License
+;;; along with GNU Emacs; see the file COPYING.  If not, write to
+;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 ;;; Commentary:
 
@@ -67,7 +65,7 @@
 ;; ../startup.el.
 
 (if (not (eq window-system 'x))
-    (error "Loading x-win.el but not compiled for X"))
+    (error "%s: Loading x-win.el but not compiled for X" (invocation-name)))
 	 
 (require 'frame)
 (require 'mouse)
@@ -76,20 +74,23 @@
 (require 'select)
 (require 'menu-bar)
 
+(defvar x-invocation-args)
+
 (defvar x-command-line-resources nil)
 
 (setq command-switch-alist
       (append '(("-bw" .	x-handle-numeric-switch)
 		("-d" .		x-handle-display)
 		("-display" .	x-handle-display)
-		("-name" .	x-handle-switch)
+		("-name" .	x-handle-name-rn-switch)
+		("-rn" .	x-handle-name-rn-switch)
 		("-T" .		x-handle-switch)
 		("-r" .		x-handle-switch)
 		("-rv" .	x-handle-switch)
 		("-reverse" .	x-handle-switch)
 		("-fn" .	x-handle-switch)
 		("-font" .	x-handle-switch)
-		("-ib" .	x-handle-switch)
+		("-ib" .	x-handle-numeric-switch)
 		("-g" .		x-handle-geometry)
 		("-geometry" .	x-handle-geometry)
 		("-fg" .	x-handle-switch)
@@ -100,7 +101,7 @@
 		("-itype" .	x-handle-switch)
 		("-i" 	.	x-handle-switch)
 		("-iconic" .	x-handle-switch)
-		("-rn" .        x-handle-rn-switch)
+		("-xrm" .       x-handle-xrm-switch)
 		("-cr" .	x-handle-switch)
 		("-vb" .	x-handle-switch)
 		("-hb" .	x-handle-switch)
@@ -124,7 +125,7 @@
     ("-cr" cursor-color)
     ("-itype" icon-type t)
     ("-i" icon-type t)
-    ("-iconic" iconic-startup t)
+    ("-iconic" visibility icon)
     ("-vb" vertical-scroll-bars t)
     ("-hb" horizontal-scroll-bars t)
     ("-bd" border-color)
@@ -155,8 +156,10 @@
 	      x-invocation-args
 	      (cdr x-invocation-args)))))
 
-;; Handle the -rn option.
-(defun x-handle-rn-switch (switch)
+;; Handle the -xrm option.
+(defun x-handle-xrm-switch (switch)
+  (or (consp x-invocation-args)
+      (error "%s: missing argument to `%s' option" (invocation-name) switch))
   (setq x-command-line-resources (car x-invocation-args))
   (setq x-invocation-args (cdr x-invocation-args)))
 
@@ -167,6 +170,18 @@
 		(x-parse-geometry (car x-invocation-args)))
 	x-invocation-args (cdr x-invocation-args)))
 
+;; Handle the -name and -rn options.  Set the variable x-resource-name
+;; to the option's operand; if the switch was `-name', set the name of
+;; the initial frame, too.
+(defun x-handle-name-rn-switch (switch)
+  (or (consp x-invocation-args)
+      (error "%s: missing argument to `%s' option" (invocation-name) switch))
+  (setq x-resource-name (car x-invocation-args)
+	x-invocation-args (cdr x-invocation-args))
+  (if (string= switch "-name")
+      (setq initial-frame-alist (cons (cons 'name x-resource-name)
+				      initial-frame-alist))))
+
 (defvar x-display-name nil
   "The X display name specifying server and X frame.")
 
@@ -515,6 +530,17 @@ This returns ARGS with the arguments that have been processed removed."
 ;;; functions and variables that we use now.
 
 (setq command-line-args (x-handle-args command-line-args))
+
+;;; Make sure we have a valid resource name.
+(or (stringp x-resource-name)
+    (let (i)
+      (setq x-resource-name (invocation-name))
+
+      ;; Change any . or * characters in x-resource-name to hyphens,
+      ;; so as not to choke when we use it in X resource queries.
+      (while (setq i (string-match "[.*]" x-resource-name))
+	(aset x-resource-name i ?-))))
+
 (x-open-connection (or x-display-name
 		       (setq x-display-name (getenv "DISPLAY")))
 		   x-command-line-resources)
@@ -529,6 +555,21 @@ This returns ARGS with the arguments that have been processed removed."
       (setq initial-frame-alist (append initial-frame-alist
 					(x-parse-geometry res-geometry)))))
 
+;; Check the reverseVideo resource.
+(let ((case-fold-search t))
+  (let ((rv (x-get-resource "reverseVideo" "ReverseVideo")))
+    (if (and rv
+	     (string-match "^\\(true\\|yes\\|on\\)$" rv))
+	(setq default-frame-alist
+	      (cons '(reverse . t) default-frame-alist)))))
+
+;; Set x-selection-timeout, measured in milliseconds.
+(let ((res-selection-timeout
+       (x-get-resource "selectionTimeout" "SelectionTimeout")))
+  (setq x-selection-timeout 5000)
+  (if res-selection-timeout
+      (setq x-selection-timeout (string-to-number res-selection-timeout))))
+
 (defun x-win-suspend-error ()
   (error "Suspending an emacs running under X makes no sense"))
 (add-hook 'suspend-hook 'x-win-suspend-error)
