@@ -16,6 +16,22 @@
 #include "XMenuInt.h"
 
 
+#ifdef EMACS_BITMAP_FILES
+#include "../src/bitmaps/dimple1.xbm"
+#include "../src/bitmaps/dimple3.xbm"
+#include "../src/bitmaps/gray1.xbm"
+#include "../src/bitmaps/gray3.xbm"
+#include "../src/bitmaps/crosswv.xbm"
+
+#include "../src/bitmaps/leftptr.xbm"
+#include "../src/bitmaps/leftpmsk.xbm"
+#include "../src/bitmaps/rtptr.xbm"
+#include "../src/bitmaps/rtpmsk.xbm"
+#include "../src/bitmaps/cntrptr.xbm"
+#include "../src/bitmaps/cntrpmsk.xbm"
+#include "../src/bitmaps/stipple.xbm"
+
+#else
 #ifndef VMS
 
 #include <X11/bitmaps/dimple1>
@@ -38,18 +54,18 @@
 #include "[-.src.bitmaps]dimple3.xbm"
 #include "[-.src.bitmaps]gray1.xbm"
 #include "[-.src.bitmaps]gray3.xbm"
-#include "[-.src.bitmaps]cross_weave.xbm"
+#include "[-.src.bitmaps]crosswv.xbm"
 
-#include "[-.src.bitmaps]left_ptr.xbm"
-#include "[-.src.bitmaps]left_ptrmsk.xbm"
-#include "[-.src.bitmaps]right_ptr.xbm"
-#include "[-.src.bitmaps]right_ptrmsk.xbm"
-#include "[-.src.bitmaps]cntr_ptr.xbm"
-#include "[-.src.bitmaps]cntr_ptrmsk.xbm"
+#include "[-.src.bitmaps]leftptr.xbm"
+#include "[-.src.bitmaps]leftpmsk.xbm"
+#include "[-.src.bitmaps]rtptr.xbm"
+#include "[-.src.bitmaps]rtpmsk.xbm"
+#include "[-.src.bitmaps]cntrptr.xbm"
+#include "[-.src.bitmaps]cntrpmsk.xbm"
 #include "[-.src.bitmaps]stipple.xbm"
 
-#endif
-
+#endif /* VMS */
+#endif /* not EMACS_BITMAP_FILES */
 
 #define DEF_FREEZE		0
 #define DEF_REVERSE		0
@@ -128,7 +144,7 @@ XMenuCreate(display, parent, def_env)
   GC inact_GC_noexpose;
 
   XColor color_def;		/* Temp color definition holder. */
-  XColor screen_def;		/* Temp screen color defintion holder */
+  XColor screen_def;		/* Temp screen color definition holder */
   XColor p_bdr_color;		/* Color of border. */
   XColor s_bdr_color;		/* Color of highlight. */
   XColor p_frg_color;		/* Color of pane foreground color. */
@@ -142,8 +158,8 @@ XMenuCreate(display, parent, def_env)
 
   Pixel p_bdr_pixel;	        /* Pane border pixel. */
   Pixel s_bdr_pixel;	        /* Selection border pixel. */
-  Pixel p_frg_pixel;	        /* Pane forground pixel. */
-  Pixel s_frg_pixel;	        /* Selection forground pixel. */
+  Pixel p_frg_pixel;	        /* Pane foreground pixel. */
+  Pixel s_frg_pixel;	        /* Selection foreground pixel. */
   Pixel bkgnd_pixel;	        /* Menu background pixel. */
 
   int *width, *height;
@@ -296,7 +312,7 @@ XMenuCreate(display, parent, def_env)
     else if (strcmp(def_val, "center") == 0) p_style = CENTER;
   }
 
-  def_val = x_get_resource_string ("paneFont", "Font");
+  def_val = x_get_resource_string ("paneFont", "PaneFont");
   if (def_val != NULL) p_fnt_name = def_val;
 
   def_val = x_get_resource_string ("paneForeground", "PaneForeground");
@@ -352,7 +368,7 @@ XMenuCreate(display, parent, def_env)
     else if (strcmp(def_val, "center") == 0) s_style = CENTER;
   }
 
-  def_val = x_get_resource_string ("selectionFont", "Font");
+  def_val = x_get_resource_string ("selectionFont", "SelectionFont");
   if (def_val != NULL) s_fnt_name = def_val;
 
   def_val = x_get_resource_string ("selectionForeground", "SelectionForeground");
