@@ -55,8 +55,10 @@ Lisp_Object Qarith_error;
 
 #include <math.h>
 
+#ifndef hpux
 /* These declarations are omitted on some systems, like Ultrix.  */
 extern double logb ();
+#endif
 
 #if defined(DOMAIN) && defined(SING) && defined(OVERFLOW)
     /* If those are defined, then this is probably a `matherr' machine. */
@@ -152,6 +154,7 @@ static char *float_error_fn_name;
     }							\
   } while (0)
 #else
+#define IN_FLOAT(d, name, num) (in_float = 1, (d), in_float = 0)
 #define IN_FLOAT2(d, name, num, num2) (in_float = 1, (d), in_float = 0)
 #endif
 
@@ -402,8 +405,9 @@ DEFUN ("expt", Fexpt, Sexpt, 2, 2, 0,
   if ((XTYPE (arg1) == Lisp_Int) && /* common lisp spec */
       (XTYPE (arg2) == Lisp_Int)) /* don't promote, if both are ints */
     {				/* this can be improved by pre-calculating */
-      int acc, x, y;		/* some binary powers of x then acumulating */
-      /* these, therby saving some time. -wsr */
+      int acc, x, y;		/* some binary powers of x then accumulating */
+      Lisp_Object val;
+
       x = XINT (arg1);
       y = XINT (arg2);
       acc = 1;
@@ -428,8 +432,8 @@ DEFUN ("expt", Fexpt, Sexpt, 2, 2, 0,
 	      y = (unsigned)y >> 1;
 	    }
 	}
-      XSET (x, Lisp_Int, acc);
-      return x;
+      XSET (val, Lisp_Int, acc);
+      return val;
     }
   f1 = (XTYPE (arg1) == Lisp_Float) ? XFLOAT (arg1)->data : XINT (arg1);
   f2 = (XTYPE (arg2) == Lisp_Float) ? XFLOAT (arg2)->data : XINT (arg2);
@@ -469,7 +473,7 @@ If second optional argument BASE is given, return log ARG using that base.")
       if (b == 10.0)
 	IN_FLOAT2 (d = log10 (d), "log", arg, base);
       else
-	IN_FLOAT2 (d = log (arg) / log (b), "log", arg, base);
+	IN_FLOAT2 (d = log (d) / log (b), "log", arg, base);
     }
   return make_float (d);
 }
@@ -656,7 +660,7 @@ DEFUN ("ceiling", Fceiling, Sceiling, 1, 1, 0,
   CHECK_NUMBER_OR_FLOAT (arg, 0);
 
   if (XTYPE (arg) == Lisp_Float)
-    IN_FLOAT (XSET (arg, Lisp_Int, ceil (XFLOAT (arg)->data)), "celing", arg);
+    IN_FLOAT (XSET (arg, Lisp_Int, ceil (XFLOAT (arg)->data)), "ceiling", arg);
 
   return arg;
 }
