@@ -1,21 +1,21 @@
 /* Markers: examining, setting and killing.
-   Copyright (C) 1985 Richard M. Stallman.
+   Copyright (C) 1985 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
-GNU Emacs is distributed in the hope that it will be useful,
-but without any warranty.  No author or distributor
-accepts responsibility to anyone for the consequences of using it
-or for whether it serves any particular purpose or works at all,
-unless he says so in writing.
+GNU Emacs is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 1, or (at your option)
+any later version.
 
-Everyone is granted permission to copy, modify and redistribute
-GNU Emacs, but only under the conditions described in the
-document "GNU Emacs copying permission notice".   An exact copy
-of the document is supposed to have been given to you along with
-GNU Emacs so that you can know how you may redistribute it all.
-It should be in a file named COPYING.  Among other things, the
-copyright notice and this notice must be preserved on all copies.  */
+GNU Emacs is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with GNU Emacs; see the file COPYING.  If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 
 #include "config.h"
@@ -28,17 +28,15 @@ DEFUN ("marker-buffer", Fmarker_buffer, Smarker_buffer, 1, 1, 0,
   "Return the buffer that MARKER points into, or nil if none.\n\
 Returns nil if MARKER points into a dead buffer.")
   (marker)
-     Lisp_Object marker;
+     register Lisp_Object marker;
 {
-  Lisp_Object buf;
-  if (XTYPE (marker) != Lisp_Marker)
-    wrong_type_argument (Qmarkerp, marker, 0);
+  register Lisp_Object buf;
+  CHECK_MARKER (marker, 0);
   if (XMARKER (marker)->buffer)
     {
-      XSETTYPE (buf, Lisp_Buffer);
-      XSETBUFFER (buf, XMARKER (marker)->buffer);
+      XSET (buf, Lisp_Buffer, XMARKER (marker)->buffer);
       /* Return marker's buffer only if it is not dead.  */
-      if (!NULL (XBUFFER (buf)->name))
+      if (!NILP (XBUFFER (buf)->name))
 	return buf;
     }
   return Qnil;
@@ -49,25 +47,22 @@ DEFUN ("marker-position", Fmarker_position, Smarker_position, 1, 1, 0,
   (marker)
      Lisp_Object marker;
 {
-  Lisp_Object pos;
-  int i;
-  struct buffer *buf;
-  struct buffer_text *text;
+  register Lisp_Object pos;
+  register int i;
+  register struct buffer *buf;
 
-  if (XTYPE (marker) != Lisp_Marker)
-    wrong_type_argument (Qmarkerp, marker, 0);
+  CHECK_MARKER (marker, 0);
   if (XMARKER (marker)->buffer)
     {
       buf = XMARKER (marker)->buffer;
       i = XMARKER (marker)->bufpos;
-      text = (buf == bf_cur) ? &bf_text : &buf->text;
 
-      if (i > text->size1 + text->gap + 1)
-	i -= text->gap;
-      else if (i > text->size1 + 1)
-	i = text->size1 + 1;
+      if (i > BUF_GPT (buf) + BUF_GAP_SIZE (buf))
+	i -= BUF_GAP_SIZE (buf);
+      else if (i > BUF_GPT (buf))
+	i = BUF_GPT (buf);
 
-      if (i < 1 || i > text->size1 + text->size2 + 1)
+      if (i < BUF_BEG (buf) || i > BUF_Z (buf))
 	abort ();
 
       XFASTINT (pos) = i;
@@ -85,41 +80,101 @@ Returns MARKER.")
   (marker, pos, buffer)
      Lisp_Object marker, pos, buffer;
 {
-  int charno;
+  register int charno;
   register struct buffer *b;
-  register struct buffer_text *text;
   register struct Lisp_Marker *m;
 
-  if (XTYPE (marker) != Lisp_Marker)
-    wrong_type_argument (Qmarkerp, marker, 0);
-  if (NULL (pos))
+  CHECK_MARKER (marker, 0);
+  /* If position is nil or a marker that points nowhere,
+     make this marker point nowhere.  */
+  if (NILP (pos)
+      || (XTYPE (pos) == Lisp_Marker && !XMARKER (pos)->buffer))
     {
       unchain_marker (marker);
-      XMARKER (marker)->buffer = 0;
       return marker;
     }
 
   CHECK_NUMBER_COERCE_MARKER (pos, 1);
-  if (NULL (buffer))
-    b = bf_cur;
+  if (NILP (buffer))
+    b = current_buffer;
   else
     {
       CHECK_BUFFER (buffer, 1);
       b = XBUFFER (buffer);
+      /* If buffer is dead, set marker to point nowhere.  */
+      if (EQ (b->name, Qnil))
+	{
+	  unchain_marker (marker);
+	  return marker;
+	}
     }
 
   charno = XINT (pos);
   m = XMARKER (marker);
 
-  if (bf_cur == b)
-    text = &bf_text;
-  else
-    text = &b->text;
+  if (charno < BUF_BEG (b))
+    charno = BUF_BEG (b);
+  if (charno > BUF_Z (b))
+    charno = BUF_Z (b);
+  if (charno > BUF_GPT (b)) charno += BUF_GAP_SIZE (b);
+  m->bufpos = charno;
 
-  if (charno < text->head_clip) charno = text->head_clip;
-  if (charno > text->size1 + text->size2 + 1 - text->tail_clip)
-    charno = text->size1 + text->size2 + 1 - text->tail_clip;
-  if (charno > text->size1 + 1) charno += text->gap;
+  if (m->buffer != b)
+    {
+      unchain_marker (marker);
+      m->chain = b->markers;
+      b->markers = marker;
+      m->buffer = b;
+    }
+  
+  return marker;
+}
+
+/* This version of Fset_marker won't let the position
+   be outside the visible part.  */
+
+Lisp_Object 
+set_marker_restricted (marker, pos, buffer)
+     Lisp_Object marker, pos, buffer;
+{
+  register int charno;
+  register struct buffer *b;
+  register struct Lisp_Marker *m;
+
+  CHECK_MARKER (marker, 0);
+  /* If position is nil or a marker that points nowhere,
+     make this marker point nowhere.  */
+  if (NILP (pos) ||
+      (XTYPE (pos) == Lisp_Marker && !XMARKER (pos)->buffer))
+    {
+      unchain_marker (marker);
+      return marker;
+    }
+
+  CHECK_NUMBER_COERCE_MARKER (pos, 1);
+  if (NILP (buffer))
+    b = current_buffer;
+  else
+    {
+      CHECK_BUFFER (buffer, 1);
+      b = XBUFFER (buffer);
+      /* If buffer is dead, set marker to point nowhere.  */
+      if (EQ (b->name, Qnil))
+	{
+	  unchain_marker (marker);
+	  return marker;
+	}
+    }
+
+  charno = XINT (pos);
+  m = XMARKER (marker);
+
+  if (charno < BUF_BEGV (b))
+    charno = BUF_BEGV (b);
+  if (charno > BUF_ZV (b))
+    charno = BUF_ZV (b);
+  if (charno > BUF_GPT (b))
+    charno += BUF_GAP_SIZE (b);
   m->bufpos = charno;
 
   if (m->buffer != b)
@@ -134,19 +189,24 @@ Returns MARKER.")
 }
 
 /* This is called during garbage collection,
- so we must be careful to ignore and preserve mark bits,
- including those in chain fields of markers.  */
+   so we must be careful to ignore and preserve mark bits,
+   including those in chain fields of markers.  */
 
 unchain_marker (marker)
-     Lisp_Object marker;
+     register Lisp_Object marker;
 {
-  Lisp_Object tail, prev, next;
+  register Lisp_Object tail, prev, next;
   register int omark;
+  register struct buffer *b;
 
-  if (!XMARKER (marker)->buffer)
+  b = XMARKER (marker)->buffer;
+  if (b == 0)
     return;
 
-  tail = XMARKER (marker)->buffer->markers;
+  if (EQ (b->name, Qnil))
+    abort ();
+
+  tail = b->markers;
   prev = Qnil;
   while (XSYMBOL (tail) != XSYMBOL (Qnil))
     {
@@ -155,8 +215,15 @@ unchain_marker (marker)
 
       if (XMARKER (marker) == XMARKER (tail))
 	{
-	  if (NULL (prev))
-	    XMARKER (marker)->buffer->markers = next;
+	  if (NILP (prev))
+	    {
+	      b->markers = next;
+	      /* Deleting first marker from the buffer's chain.
+		 Crash if new first marker in chain does not say
+		 it belongs to this buffer.  */
+	      if (!EQ (next, Qnil) && b != XMARKER (next)->buffer)
+		abort ();
+	    }
 	  else
 	    {
 	      omark = XMARKBIT (XMARKER (prev)->chain);
@@ -178,18 +245,16 @@ marker_position (marker)
   register struct Lisp_Marker *m = XMARKER (marker);
   register struct buffer *buf = m->buffer;
   register int i = m->bufpos;
-  register struct buffer_text *text
-    = (buf == bf_cur) ? &bf_text : &buf->text;
 
   if (!buf)
     error ("Marker does not point anywhere");
 
-  if (i > text->size1 + text->gap + 1)
-    i -= text->gap;
-  else if (i > text->size1 + 1)
-    i = text->size1 + 1;
+  if (i > BUF_GPT (buf) + BUF_GAP_SIZE (buf))
+    i -= BUF_GAP_SIZE (buf);
+  else if (i > BUF_GPT (buf))
+    i = BUF_GPT (buf);
 
-  if (i < 1 || i > text->size1 + text->size2 + 1)
+  if (i < BUF_BEG (buf) || i > BUF_Z (buf))
     abort ();
 
   return i;
@@ -200,23 +265,25 @@ DEFUN ("copy-marker", Fcopy_marker, Scopy_marker, 1, 1, 0,
 If argument is a number, makes a new marker pointing\n\
 at that position in the current buffer.")
   (marker)
-     Lisp_Object marker;
+     register Lisp_Object marker;
 {
-  Lisp_Object new;
+  register Lisp_Object new;
 
-  if (XTYPE (marker) == Lisp_Int
-      || XTYPE (marker) == Lisp_Marker)
+  while (1)
     {
-      new = Fmake_marker ();
-      Fset_marker (new, marker,
-		   XTYPE (marker) == Lisp_Marker
-		   ? Fmarker_buffer (marker)
-		   : Qnil);
+      if (XTYPE (marker) == Lisp_Int
+	  || XTYPE (marker) == Lisp_Marker)
+	{
+	  new = Fmake_marker ();
+	  Fset_marker (new, marker,
+		       ((XTYPE (marker) == Lisp_Marker)
+			? Fmarker_buffer (marker)
+			: Qnil));
+	  return new;
+	}
+      else
+	marker = wrong_type_argument (Qinteger_or_marker_p, marker);
     }
-  else
-    wrong_type_argument (Qinteger_or_marker_p, marker, 0);
-
-  return new;
 }
 
 syms_of_marker ()
@@ -224,6 +291,5 @@ syms_of_marker ()
   defsubr (&Smarker_position);
   defsubr (&Smarker_buffer);
   defsubr (&Sset_marker);
-  defalias (&Sset_marker, "move-marker");
   defsubr (&Scopy_marker);
 }
