@@ -235,7 +235,11 @@ int update_tick;
 /* We could get this from param.h, but better not to depend on finding that.
    And better not to risk that it might define other symbols used in this
    file.  */
+#ifdef FD_SETSIZE
+#define MAXDESC FD_SETSIZE
+#else
 #define MAXDESC 64
+#endif
 #define SELECT_TYPE fd_set
 #else /* no FD_SET */
 #define MAXDESC 32
@@ -287,7 +291,7 @@ update_status (p)
   p->raw_status_high = Qnil;
 }
 
-/*  Convert a process status work in Unix format to 
+/*  Convert a process status word in Unix format to 
     the list that we use internally.  */
 
 Lisp_Object
@@ -1074,7 +1078,7 @@ start_process_unwind (proc)
     abort ();
 
   /* Was PROC started successfully?  */
-  if (XPROCESS (proc)->pid <= 0)
+  if (XINT (XPROCESS (proc)->pid) <= 0)
     remove_process (proc);
 
   return Qnil;
@@ -1261,7 +1265,7 @@ create_process (process, new_argv, current_dir)
 	  ioctl (xforkin, TIOCSCTTY, 0);
 #endif
 #else /* not HAVE_SETSID */
-#ifdef USG
+#if defined (USG) && !defined (IRIX)
 	/* It's very important to call setpgrp() here and no time
 	   afterwards.  Otherwise, we lose our controlling tty which
 	   is set when we open the pty. */
@@ -1474,7 +1478,7 @@ Fourth arg SERVICE is name of the service desired, or an integer\n\
     report_file_error ("error creating socket", Fcons (name, Qnil));
 
  loop:
-  if (connect (s, &address, sizeof address) == -1)
+  if (connect (s, (struct sockaddr *) &address, sizeof address) == -1)
     {
       int xerrno = errno;
       if (errno == EINTR)
@@ -1654,32 +1658,34 @@ static int waiting_for_user_input_p;
 /* Read and dispose of subprocess output while waiting for timeout to
    elapse and/or keyboard input to be available.
 
-   time_limit is:
+   TIME_LIMIT is:
      timeout in seconds, or
      zero for no limit, or
      -1 means gobble data immediately available but don't wait for any.
 
-   microsecs is:
-     an additional duration to wait (if time_limit is greater than
-     zero), specified in millisec.
+   MICROSECS is:
+     an additional duration to wait, measured in microseconds.
+     If this is nonzero and time_limit is 0, then the timeout
+     consists of MICROSECS only.
 
-   read_kbd is a lisp value:
+   READ_KBD is a lisp value:
      0 to ignore keyboard input, or
      1 to return when input is available, or
      -1 meaning caller will actually read the input, so don't throw to
        the quit handler, or
-     a cons cell, meaning wait wait until its car is non-nil, or
+     a cons cell, meaning wait wait until its car is non-nil
+       (and gobble terminal input into the buffer if any arrives), or
      a process object, meaning wait until something arrives from that
        process.  The return value is true iff we read some input from
        that process.
 
-   do_display != 0 means redisplay should be done to show subprocess
+   DO_DISPLAY != 0 means redisplay should be done to show subprocess
    output that arrives.
 
-   If read_kbd is a pointer to a struct Lisp_Process, then the
+   If READ_KBD is a pointer to a struct Lisp_Process, then the
      function returns true iff we received input from that process
      before the timeout elapsed.
-   Otherwise, return true iff we recieved input from any process.  */
+   Otherwise, return true iff we received input from any process.  */
 
 wait_reading_process_input (time_limit, microsecs, read_kbd, do_display)
      int time_limit, microsecs;
@@ -1795,7 +1801,7 @@ wait_reading_process_input (time_limit, microsecs, read_kbd, do_display)
       /* Wait till there is something to do */
 
       Available = input_wait_mask;
-      if (! XINT (read_kbd))
+      if (! XINT (read_kbd) && wait_for_cell == 0)
 	FD_CLR (0, &Available);
 
       /* If frame size has changed or the window is newly mapped,
@@ -1874,7 +1880,8 @@ wait_reading_process_input (time_limit, microsecs, read_kbd, do_display)
       /* If there is any, return immediately
 	 to give it higher priority than subprocesses */
 
-      if (XINT (read_kbd) && detect_input_pending ())
+      if ((XINT (read_kbd) || wait_for_cell)
+	  && detect_input_pending ())
 	{
 	  swallow_events ();
 	  if (detect_input_pending ())
@@ -1900,7 +1907,7 @@ wait_reading_process_input (time_limit, microsecs, read_kbd, do_display)
 
       /* If checking input just got us a size-change event from X,
 	 obey it now if we should.  */
-      if (XINT (read_kbd))
+      if (XINT (read_kbd) || wait_for_cell)
 	do_pending_window_change ();
 
       /* Check for data from a process or a command channel */
@@ -2071,9 +2078,16 @@ read_process_output (proc, channel)
 	 hitting ^G when a filter happens to be running won't screw
 	 it up.  */
       int count = specpdl_ptr - specpdl;
+      Lisp_Object odeactivate;
+
+      odeactivate = Vdeactivate_mark;
+
       specbind (Qinhibit_quit, Qt);
       call2 (outstream, proc, make_string (chars, nchars));
 
+      /* Handling the process output should not deactivate the mark.  */
+      Vdeactivate_mark = odeactivate;
+
 #ifdef VMS
       start_vms_process_read (vs);
 #endif
@@ -2086,6 +2100,9 @@ read_process_output (proc, channel)
     {
       Lisp_Object old_read_only;
       Lisp_Object old_begv, old_zv;
+      Lisp_Object odeactivate;
+
+      odeactivate = Vdeactivate_mark;
 
       Fset_buffer (p->buffer);
       opoint = point;
@@ -2130,6 +2147,9 @@ read_process_output (proc, channel)
       if (XFASTINT (old_begv) != BEGV || XFASTINT (old_zv) != ZV)
 	Fnarrow_to_region (old_begv, old_zv);
 
+      /* Handling the process output should not deactivate the mark.  */
+      Vdeactivate_mark = odeactivate;
+
       current_buffer->read_only = old_read_only;
       SET_PT (opoint);
       set_buffer_internal (old);
@@ -2783,7 +2803,7 @@ sigchld_handler (signo)
 	    synch_process_retcode = WRETCODE (w);
 	  else if (WIFSIGNALED (w))
 #ifndef VMS
-	    synch_process_death = sys_siglist[WTERMSIG (w)];
+	    synch_process_death = (char *) sys_siglist[WTERMSIG (w)];
 #else
 	    synch_process_death = sys_errlist[WTERMSIG (w)];
 #endif
@@ -3085,7 +3105,7 @@ extern int frame_garbaged;
    do_display != 0 means redisplay should be done to show subprocess
    output that arrives.  This version of the function ignores it.
 
-   Return true iff we recieved input from any process.  */
+   Return true iff we received input from any process.  */
 
 int
 wait_reading_process_input (time_limit, microsecs, read_kbd, do_display)
@@ -3173,8 +3193,10 @@ wait_reading_process_input (time_limit, microsecs, read_kbd, do_display)
 	/* System sometimes fails to deliver SIGIO.  */
 	kill (getpid (), SIGIO);
 #endif
+#ifdef SIGIO
       if (XINT (read_kbd) && interrupt_input && (waitchannels & 1))
 	kill (0, SIGIO);
+#endif
 
       /* If we have timed out (nfds == 0) or found some input (nfds > 0),
 	 we should exit.  */
