@@ -92,6 +92,9 @@ static Lisp_Object last_arrow_position, last_arrow_string;
 /* Nonzero if overlay arrow has been displayed once in this window.  */
 static int overlay_arrow_seen;
 
+/* Nonzero means highlight the region even in nonselected windows.  */
+static int highlight_nonselected_windows;
+
 /* If cursor motion alone moves point off frame,
    Try scrolling this many lines up or down if that will bring it back.  */
 int scroll_step;
@@ -309,7 +312,7 @@ echo_area_display ()
 
   if (frame_garbaged)
     {
-      Fredraw_display ();
+      redraw_garbaged_frames ();
       frame_garbaged = 0;
     }
 
@@ -329,7 +332,8 @@ echo_area_display ()
       {
 	int i;
 
-	for (i = vpos + 1; i < vpos + XWINDOW (minibuf_window)->height; i++)
+	for (i = vpos + 1;
+	     i < vpos + XFASTINT (XWINDOW (minibuf_window)->height); i++)
 	  {
 	    get_display_line (f, i, 0);
 	    display_string (XWINDOW (minibuf_window), vpos,
@@ -391,7 +395,7 @@ redisplay ()
 
   if (frame_garbaged)
     {
-      Fredraw_display ();
+      redraw_garbaged_frames ();
       frame_garbaged = 0;
     }
 
@@ -431,8 +435,9 @@ redisplay ()
   if (((!NILP (Vtransient_mark_mode)
 	&& !NILP (XBUFFER (w->buffer)->mark_active))
        != !NILP (w->region_showing))
-      || !EQ (w->region_showing,
-	      Fmarker_position (XBUFFER (w->buffer)->mark)))
+      || (!NILP (w->region_showing)
+	  && !EQ (w->region_showing,
+		  Fmarker_position (XBUFFER (w->buffer)->mark))))
     this_line_bufpos = -1;
 
   tlbufpos = this_line_bufpos;
@@ -947,6 +952,8 @@ redisplay_window (window, just_this_one)
   else if (just_this_one && !MINI_WINDOW_P (w)
 	   && point >= startp
 	   && XFASTINT (w->last_modified)
+	   /* or else vmotion on first line won't work.  */
+	   && ! NILP (w->start_at_line_beg)
 	   && ! EQ (w->window_end_valid, Qnil)
 	   && do_id && !clip_changed
 	   && !blank_end_of_window
@@ -1082,7 +1089,7 @@ done:
 	start = end = whole = 0;
 
       /* Indicate what this scroll bar ought to be displaying now.  */
-      (*set_vertical_scroll_bar_hook) (w, end - start, whole, start - 1);
+      (*set_vertical_scroll_bar_hook) (w, end - start, whole, start);
 
       /* Note that we actually used the scroll bar attached to this window,
 	 so it shouldn't be deleted at the end of redisplay.  */
@@ -1594,9 +1601,11 @@ copy_rope (t, s, from, face)
 
   while (n--)
     {
-      if (t >= s) *t = MAKE_GLYPH (GLYPH_CHAR (*f),
-				   (GLYPH_FACE (*f)
-				    ? GLYPH_FACE (*f)
+      int glyph = XFASTINT (*f);
+
+      if (t >= s) *t = MAKE_GLYPH (GLYPH_CHAR (glyph),
+				   (GLYPH_FACE (glyph)
+				    ? GLYPH_FACE (glyph)
 				    : face));
       ++t;
       ++f;
@@ -1620,9 +1629,11 @@ copy_part_of_rope (t, s, from, len, face)
 
   while (n--)
     {
-      if (t >= s) *t = MAKE_GLYPH (GLYPH_CHAR (*f),
-				   (GLYPH_FACE (*f)
-				    ? GLYPH_FACE (*f)
+      int glyph = XFASTINT (*f);
+
+      if (t >= s) *t = MAKE_GLYPH (GLYPH_CHAR (glyph),
+				   (GLYPH_FACE (glyph)
+				    ? GLYPH_FACE (glyph)
 				    : face));
       ++t;
       ++f;
@@ -1662,7 +1673,7 @@ display_text_line (w, start, vpos, hpos, taboffset)
   register unsigned char *p;
   GLYPH *endp;
   register GLYPH *startp;
-  register GLYPH *p1prev;
+  register GLYPH *p1prev = 0;
   FRAME_PTR f = XFRAME (w->frame);
   int tab_width = XINT (current_buffer->tab_width);
   int ctl_arrow = !NILP (current_buffer->ctl_arrow);
@@ -1724,8 +1735,9 @@ display_text_line (w, start, vpos, hpos, taboffset)
 
   /* Show where to highlight the region.  */
   if (highlight_region && XMARKER (current_buffer->mark)->buffer != 0
-      /* Highlight only in selected window.  */
-      && w == XWINDOW (FRAME_SELECTED_WINDOW (f)))
+      /* Maybe highlight only in selected window.  */
+      && (highlight_nonselected_windows
+	  || w == XWINDOW (selected_window)))
     {
       region_beg = marker_position (current_buffer->mark);
       if (PT < region_beg)
@@ -1928,10 +1940,16 @@ display_text_line (w, start, vpos, hpos, taboffset)
   /* by backing up over it */
   if (p1 > endp)
     {
-      /* Start the next line with that same character */
-      pos--;
-      /* but at a negative hpos, to skip the columns output on this line.  */
-      val.hpos += p1prev - endp;
+      /* Don't back up if we never actually displayed any text.
+	 This occurs when the minibuffer prompt takes up the whole line.  */
+      if (p1prev)
+	{
+	  /* Start the next line with that same character */
+	  pos--;
+	  /* but at negative hpos, to skip the columns output on this line.  */
+	  val.hpos += p1prev - endp;
+	}
+
       /* Keep in this line everything up to the continuation column.  */
       p1 = endp;
     }
@@ -2045,14 +2063,17 @@ display_text_line (w, start, vpos, hpos, taboffset)
       unsigned char *p = XSTRING (Voverlay_arrow_string)->data;
       int i;
       int len = XSTRING (Voverlay_arrow_string)->size;
+      int arrow_end;
 
       if (len > width)
 	len = width;
       for (i = 0; i < len; i++)
 	startp[i] = p[i];
-      if (desired_glyphs->used[vpos] <
-	  (len + startp - desired_glyphs->glyphs[vpos]))
-	desired_glyphs->used[vpos] = len + startp - desired_glyphs->glyphs[vpos];
+
+      /* Bug in SunOS 4.1.1 compiler requires this intermediate variable.  */
+      arrow_end = (startp - desired_glyphs->glyphs[vpos]) + len;
+      if (desired_glyphs->used[vpos] < arrow_end)
+	desired_glyphs->used[vpos] = arrow_end;
 
       overlay_arrow_seen = 1;
     }
@@ -2836,6 +2857,10 @@ If this is zero, point is always centered after it moves off frame.");
   DEFVAR_INT ("line-number-display-limit", &line_number_display_limit,
     "*Maximum buffer size for which line number should be displayed.");
   line_number_display_limit = 1000000;
+
+  DEFVAR_BOOL ("highlight-nonselected-windows", &highlight_nonselected_windows,
+    "*Non-nil means highlight region even in nonselected windows.");
+  highlight_nonselected_windows = 1;
 }
 
 /* initialize the window system */
